#!/bin/echo It can be only inclusion. Incorrect start

# Use this for include.
# [ -z "${_NSG_FUNCTIONS}" ] && { . /usr/local/share/nsg/lib/functions.sh || do_it_if_error; }

_NSG_FUNCTIONS=1

# Exit status codes for system programs.
#
# This include file attempts to categorize possible error
# exit statuses for system programs, notably delivermail
# and the Berkeley network.
#
# Error numbers begin at EX__BASE to reduce the possibility of
# clashing with other exit statuses that random programs may
# already return.  The meaning of the codes is approximately
# as follows:
#
# EX_USAGE -- The command was used incorrectly, e.g., with
# 	the wrong number of arguments, a bad flag, a bad
# 	syntax in a parameter, or whatever.
# EX_DATAERR -- The input data was incorrect in some way.
# 	This should only be used for user's data & not
# 	system files.
# EX_NOINPUT -- An input file (not a system file) did not
# 	exist or was not readable.  This could also include
#  	errors like "No message" to a mailer (if it cared
#  	to catch it).
# EX_NOUSER -- The user specified did not exist.  This might
# 	be used for mail addresses or remote logins.
# EX_NOHOST -- The host specified did not exist.  This is used
# 	in mail addresses or network requests.
# EX_UNAVAILABLE -- A service is unavailable.  This can occur
# 	if a support program or file does not exist.  This
# 	can also be used as a catchall message when something
# 	you wanted to do doesn't work, but you don't know
# 	why.
# EX_SOFTWARE -- An internal software error has been detected.
# 	This should be limited to non-operating system related
# 	errors as possible.
# EX_OSERR -- An operating system error has been detected.
# 	This is intended to be used for such things as "cannot
# 	fork", "cannot create pipe", or the like.  It includes
# 	things like getuid returning a user that does not
# 	exist in the passwd file.
# EX_OSFILE -- Some system file (e.g., /etc/passwd, /etc/utmp,
# 	etc.) does not exist, cannot be opened, or has some
# 	sort of error (e.g., syntax error).
# EX_CANTCREAT -- A (user specified) output file cannot be
# 	created.
# EX_IOERR -- An error occurred while doing I/O on some file.
# EX_TEMPFAIL -- temporary failure, indicating something that
# 	is not really an error.  In sendmail, this means
# 	that a mailer (e.g.) could not create a connection,
# 	and the request should be reattempted later.
# EX_PROTOCOL -- the remote system returned something that
# 	was "not possible" during a protocol exchange.
# EX_NOPERM -- You did not have sufficient permission to
# 	perform the operation.  This is not intended for
# 	file system problems, which should use NOINPUT or
# 	CANTCREAT, but rather for higher level permissions.
#
#define EX_OK           0       /* successful termination */
#define EX__BASE        64      /* base value for error messages */
#define EX_USAGE        64      /* command line usage error */
#define EX_DATAERR      65      /* data format error */
#define EX_NOINPUT      66      /* cannot open input */
#define EX_NOUSER       67      /* addressee unknown */
#define EX_NOHOST       68      /* host name unknown */
#define EX_UNAVAILABLE  69      /* service unavailable */
#define EX_SOFTWARE     70      /* internal software error */
#define EX_OSERR        71      /* system error (e.g., can't fork) */
#define EX_OSFILE       72      /* critical OS file missing */
#define EX_CANTCREAT    73      /* can't create (user) output file */
#define EX_IOERR        74      /* input/output error */
#define EX_TEMPFAIL     75      /* temp failure; user is invited to retry */
#define EX_PROTOCOL     76      /* remote error in protocol */
#define EX_NOPERM       77      /* permission denied */
#define EX_CONFIG       78      /* configuration error */
#define EX__MAX 	78     	/* maximum listed value */
#
# Use one of this for include.
# [ -z "${__SYSEXITS}" ] && { sysexits || do_it_if_error; }

sysexits() {
    __SYSEXITS=1
    
    EX_OK=0
    
    EX__BASE=64
    
    EX_USAGE=64 	
    EX_DATAERR=65
    EX_NOINPUT=66
    EX_NOUSER=67
    EX_NOHOST=68
    EX_UNAVAILABLE=69
    EX_SOFTWARE=70
    EX_OSERR=71
    EX_OSFILE=72
    EX_CANTCREAT=73
    EX_IOERR=74
    EX_TEMPFAIL=75
    EX_PROTOCOL=76
    EX_NOPERM=77
    EX_CONFIG=78
    
    EX__MAX=78
}
[ -z "${__SYSEXITS}" ] && sysexits

# Markers: (--) probed, (**) from config file, (==) default setting,
# (++) from command line, (!!) notice, (II) informational,
# (WW) warning, (EE) error, (NI) not implemented, (??) unknown,
# (DD) debug.

eerr() {
    echo -E "(EE) $*" >&2
    sleep 20
}

ewarn() {
    echo -E "(WW) $*"
}

einfo() {
    echo -E "(II) $*"
}

edebug() {
    echo -E "(DD) $*" >&2
}

#isalnum
isalnum() {
    local l_i
    
    for l_i; do
	if echo "${l_i}" | grep -qv -e '^[[:alnum:]]*$'; then
	    eerr "isalnum(): '${l_i}' contains not an alphanumeric character."
	    return ${EX_DATAERR}
	fi
    done
}

#isalpha
#isblank
#iscntrl

#isdigit
isdigit() {
    local l_i
    for l_i; do
	if echo "${l_i}" | grep -qv -e '^[[:digit:]]*$'; then
	    eerr "isidigit(): '${l_i}' contains not a digit (0 through 9) character."
	    return ${EX_DATAERR}
	fi
    done
}

#isgraph
#islower
#isprint
isprint() {
    local l_i
    for l_i; do
	if echo "${l_i}" | grep -qv -e '^[[:print:]]*$'; then
	    eerr "isprint(): '${l_i}' contains not a printable character including space."
	    return ${EX_DATAERR}
	fi
    done
}

#ispunct
#isspace
#isupper
#isxdigit

isidentifier() {
    local l_i l_v

    for l_i; do
	if eval l_v="\${${l_i}}"; then
	    continue
	else
	    eerr "isidentifier(): '${l_i}' not a valid identifier."
	    return ${EX_DATAERR}
	fi
    done
}

setflag() {
    local l_name l_flag

    l_name="$1"
    l_flag="$2"

    eval "${l_name}"=\"\${l_flag}\"
}

addflag() {
    local l_name l_flag l_w

    l_name="$1"
    l_flag="$2"
    
    for l_w in $(eval echo -E \"\${${l_name}}\"); do
	case "${l_w}" in
	    ("${l_flag}")
	    return ${EX_OK}
	    ;;
	    (*)
	    continue
	    ;;
	esac
    done
    eval "${l_name}"=\"\${${l_name}} \${l_flag}\"
}

delflag() {
    local l_name l_flag l_w l_new

    l_name="$1"
    l_flag="$2"
    
    l_new=""
    for l_w in $(eval echo -E \"\${${l_name}}\"); do
	case "${l_w}" in
	    ("${l_flag}")
	    continue
	    ;;
	    (*)
	    l_new="${l_new} ${l_w}"
	    ;;
	esac
    done
    
    eval "${l_name}"=\"\${l_new}\"
}

findflag() {
    local l_name l_flag l_w

    l_name="$1"
    l_flag="$2"
    
    for l_w in $(eval echo -E \"\${${l_name}}\"); do
	case "${l_w}" in
	    ("${l_flag}")
	    return ${EX_OK}
	    ;;
	    (*)
	    continue
	    ;;
	esac
    done
    
    # not found
    return 1
}

pushflag() {
    eval "$1"="\"\$$1 $2\""
}

popflag() {
    eval set -- "\$$1 \$1"
    eval eval "\$$#"="\\\"\\\""
    
    while true; do
	if [ "$#" -le "2" ]; then
	    break
	else
	    eval eval "\$$#"="\\\"\\\$\$$# $1\\\""
	    shift
	fi
    done
    
    if [ "$#" -eq "2" ]; then
	echo "$1"
    else
	# empty
        return 1
    fi
}

setflags() {
    local l_name l_flag
    
    l_name="$1"
    l_flag="$2"
    shift 2
    
    setflag "$l_name" "$l_flag"
    for l_flag; do
	addflag "$l_name" "$l_flag"
    done
}

addflags() {
    local l_name l_flag
    
    l_name="$1"
    shift
    
    for l_flag; do
	addflag "$l_name" "$l_flag"
    done
}

addflags() {
    local l_name l_flag
    
    l_name="$1"
    shift
    
    for l_flag; do
	delflag "$l_name" "$l_flag"
    done
}

findflags_and() {
    local l_name l_flag
    
    l_name="$1"
    shift
    
    for l_flag; do
	if findflag "$l_name" "$l_flag"; then
	    continue
	else
	    return
	fi
    done
}

findflags_or() {
    local l_name l_flag
    
    l_name="$1"
    shift
    
    for l_flag; do
	if findflag "$l_name" "$l_flag"; then
	    return ${EX_OK}
	else
	    continue
	fi
    done

    # not found
    return 1
}
