#!/bin/echo It can be only inclusion. Incorrect start

# Use this for include.
# [ -z "${_NSG_ROOTFS}" ] && { . /usr/local/share/nsg/lib/rootfs.sh || do_it_if_error; }

_NSG_ROOTFS=1

[ -z "${_NSG_FUNCTIONS}" ] && { . /usr/local/share/nsg/lib/functions.sh || return 69; }

nsg_getmtd() {
    local l_name l_s

    l_name="$1"
    l_s="$(cat /proc/mtd | sed -n -e ':start;s/^mtd\([^:]*\).* '"$l_name"'$/\1/p;t end;n;b start;:end;q')"

    if [ -n "$l_s" ]; then
	echo -E "/dev/mtd/${l_s}"
	return ${EX_OK}
    else
	return ${EX_UNAVAILABLE}
    fi
    
    return ${EX_SOFTWARE}
}

nsg_getrootdev() {
    local l_name l_s

    l_name="$1"
    case "$l_name" in
	(nftl)
	if l_s="$(nsg_getmtd "\"DiskOnChip 2000.*\"")"; then
	    echo -E "${l_s}"
	    return ${EX_OK}
	else
	    return ${EX_UNAVAILABLE}
	fi
	;;
	(flash)
	if l_s="$(nsg_getmtd "\"NSG Software\"")"; then
	    echo -E "${l_s}"
	    return ${EX_OK}
	else
	    return ${EX_UNAVAILABLE}
	fi
	;;
	(nfs)
	echo -E "/dev/nfs"
	return ${EX_OK}
	;;
	(*)
	return ${EX_USAGE}
	;;
    esac

    return ${EX_SOFTWARE}
}

nsg_getconfigdev() {
    local l_s

    if l_s="$(nsg_getmtd "\"NSG Config\"")"; then
	echo -E "${l_s}"
	return ${EX_OK}
    else
	return ${EX_UNAVAILABLE}
    fi

    return ${EX_SOFTWARE}
}

nsg_findroot() {
    local l_name l_err

    l_name="$1"

    if nsg_getrootdev "nftl"; then
	addflag "$l_name" "nftl"
    fi
    if nsg_getrootdev "flash"; then 
	addflag "$l_name" "flash"
    fi
}

nsg_nftlroot() {
    l_return_hook() {
	local l_i l_err

	l_err=${EX_OK}
	for l_i; do
	    case "$l_i" in
		(0)
		:
		;;
		(1)
		if umount -r /mnt/nsg; then
		    :
		else
		    eerr "nsg_nftlroot(): Unmounting file system from '/mnt/nsg' failed."
		    l_err=${EX_UNAVAILABLE}
		fi
		;;
		(*)
		eerr "nsg_nftlroot(): l_return_hook(): Internal error."
		l_err=${EX_SOFTWARE}
		;;
	    esac
	done
	return ${l_err}
    }

    local l_nsgrootdev
    
    if l_nsgrootdev="$(nsg_getrootdev "nftl")"; then
	:
    else
	eerr "nsg_nftlroot(): Can't get rootfs device."
	l_return_hook 0
	return ${EX_UNAVAILABLE}
    fi
    l_nsgrootdev="$(sed -n -e 's/^[[:space:]]*93[[:space:]]*0[[:space:]]*[[:digit:]]*[[:space:]]*\(.*\)[[:space:]]*$/\/dev\/\1/p;T;q' /proc/partitions)"
    if [ -z "${l_nsgrootdev}" ]; then
	eerr "nsg_nftlroot(): Nftl patrtition needs format."
	l_return_hook 0
	return ${EX_UNAVAILABLE}
    fi
    if [ ! -b "${l_nsgrootdev}" ]; then
	eerr "nsg_nftlroot(): File '${l_nsgrootdev}' not found or is not block special."
	l_return_hook 0
	return ${EX_UNAVAILABLE}
    fi
    
    if [ ! -d /mnt/nsg/ ]; then 
	ewarn "nsg_nftlroot(): Mountpoint '/mnt/nsg' not found."
	if mkdir -p /mnt/nsg; then
	    :
	else
	    eerr "nsg_nftlroot(): Can't create directory '/mnt/nsg'."
	    l_return_hook 0
	    return ${EX_UNAVAILABLE}
	fi
    fi
    
    if mount -o ro ${l_nsgrootdev} /mnt/nsg; then
	:
    else
	eerr "nsg_nftlroot(): Mounting '${l_nsgrootdev}' on '/mnt/nsg' failed."
	l_return_hook 0
	return ${EX_UNAVAILABLE}
    fi

# step 1
    if mount -o remount,rw /mnt/nsg; then
	chmod "u=rwx,go=rx" /mnt/nsg
    else
	eerr "nsg_nftlroot(): Remounting file system read-write on '/mnt/nsg' failed."
	l_return_hook 1 0
	return ${EX_UNAVAILABLE}
    fi

    if findflag "nsgdefconfig" "true"; then
	ewarn "nsg_nftlroot(): Default config is set up manually."
	
	tar --no-same-owner -xz -f /mnt/nsg/mnt/defetc.tar.gz -C /mnt/nsg/etc > /dev/null 2>&1
	if [ "$?" -ne "0" ]; then
	    eerr "nsg_nftlroot(): NSG default config is corrupt or not found."
	    l_return_hook 1 0
	    return ${EX_UNAVAILABLE}
	fi
	
	if [ ! -d /mnt/nsg/etc ]; then
	    mkdir -p /mnt/nsg/etc
	    if [ "$?" -ne "0" ]; then
		eerr "nsg_nftlroot(): Can't create directory '/mnt/nsg/etc'."
		l_return_hook 1 0
		return ${EX_UNAVAILABLE}
	    fi
	fi
	
	cd /mnt/nsg/etc
	if [ "$?" -ne "0" ]; then
	    eerr "nsg_nftlroot(): Can't change directory to '/mnt/nsg/etc'."
	    l_return_hook 1 0
	    return ${EX_UNAVAILABLE}
	fi
	
	if [ "$(pwd -P)" = "/mnt/nsg/etc" ]; then
	    rm -rf . > /dev/null 2>&1
	else
	    eerr "nsg_nftlroot(): Current working directory is not '/mnt/nsg/etc'."
	    l_return_hook 1 0
	    return ${EX_UNAVAILABLE}
	fi
	
	findflag "nsgverbose" "true" && \
	    einfo "nsg_nftlroot(): Extracting default config. Please wait..."
	tar --no-same-owner -xz -f /mnt/nsg/mnt/defetc.tar.gz -C /mnt/nsg/etc
	if [ "$?" -ne "0" ]; then
	    eerr "nsg_nftlroot(): NSG default config is corrupt or not found."
	    l_return_hook 1 0
	    return ${EX_UNAVAILABLE}
	fi
    fi

    if mount -t tmpfs none /mnt/nsg/var; then
	chmod "ugo=rx" /mnt/nsg/proc
	chmod "u=rwx,go=rx" /mnt/nsg/var
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/home
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/home/user
	chown 1000:1000 /mnt/nsg/var/home/user
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/empty
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/lock
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/log
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/run
	mkdir -m "a=rwx" /mnt/nsg/var/tmp
    else
	eerr "nsg_nftlroot(): Mounting tmpfs on '/mnt/nsg/var' failed."
	l_return_hook 1 0
	return ${EX_UNAVAILABLE}
    fi
    
    if /sbin/depmod -a -b /mnt/nsg; then
	:
    else
	ewarn "nsg_nftlroot(): Depmod can't create dependency file."
    fi

    if [ -e /proc/sys/nsg/.sys/rootdev ]; then
	echo -E "${l_nsgrootdev}" > /proc/sys/nsg/.sys/rootdev
    else
	ewarn "nsg_nftlroot(): File '/proc/sys/nsg/.sys/rootdev' not found."
    fi

    return ${EX_OK}
}

nsg_flashroot() {
    l_return_hook() {
	local l_i l_err

	l_err=${EX_OK}
	for l_i; do
	    case "$l_i" in
		(0)
		:
		;;
		(1)
		if umount -r /mnt/nsg; then
		    :
		else
		    eerr "nsg_flashroot(): Unmounting file system from '/mnt/nsg' failed."
		    l_err=${EX_UNAVAILABLE}
		fi
		;;
		(2)
		if umount -r /mnt/nsg/etc; then
		    :
		else
		    eerr "nsg_flashroot(): Unmounting file system from '/mnt/nsg/etc' failed."
		    l_err=${EX_UNAVAILABLE}
		fi
		;;
		(3)
		if umount -r /mnt/nsg/var; then
		    :
		else
		    eerr "nsg_flashroot(): Unmounting file system from '/mnt/nsg/var' failed."
		    l_err=${EX_UNAVAILABLE}
		fi
		;;
		(*)
		eerr "nsg_flashroot(): l_return_hook(): Internal error."
		l_err=${EX_SOFTWARE}
		;;
	    esac
	done
	return ${l_err}
    }

    local l_nsgrootdev l_nsgconfigdev l_name l_nsgdefconfig
    local l_NSGROOTOFFSET
    
    # (0x00400000-0x00080000)/64k=56
    l_NSGROOTOFFSET=56
    
    if l_nsgrootdev="$(nsg_getrootdev "flash")"; then
	:
    else
	eerr "nsg_flashroot(): Can't get rootfs device."
	l_return_hook 0
	return ${EX_UNAVAILABLE}
    fi
    
    if [ ! -d /mnt/nsg/ ]; then 
	ewarn "nsg_flashroot(): Mountpoint '/mnt/nsg' not found."
	if mkdir -p /mnt/nsg; then
	    :
	else
	    eerr "nsg_flashroot(): Can't create directory '/mnt/nsg'."
	    l_return_hook 0
	    return ${EX_UNAVAILABLE}
	fi
    fi

    if mount -o ro,size=24m -t tmpfs none /mnt/nsg; then
	:
    else
	eerr "nsg_flashroot(): Mounting tmpfs on '/mnt/nsg' failed."
	l_return_hook 0
	return ${EX_UNAVAILABLE}
    fi

# step 1
    if mount -o remount,rw none /mnt/nsg; then
	chmod "u=rwx,go=rx" /mnt/nsg
    else
	eerr "nsg_flashroot(): Remounting tmpfs read-write on '/mnt/nsg' failed."
	l_return_hook 1 0
	return ${EX_UNAVAILABLE}
    fi

    [ "${nsgverbose}" = "true" ] && 
    einfo "nsg_flashroot(): Extracting rootfs. Please wait..."
    dd if="$l_nsgrootdev" bs=64k skip=${l_NSGROOTOFFSET} | \
	tar --no-same-owner -xz -f - -C /mnt/nsg
    if [ "$?" -ne "0" ]; then
	eerr "nsg_flashroot(): Data of the rootfs on '${l_nsgrootdev}' is corrupt."
	l_return_hook 1 0
	return ${EX_UNAVAILABLE}
    fi

    if l_nsgconfigdev=`nsg_getconfigdev`; then
	:
    else
	eerr "nsg_flashroot(): Can't get nsgconfig device."
	l_return_hook 1 0
	return ${EX_UNAVAILABLE}
    fi
    
    if mount -t tmpfs none /mnt/nsg/etc; then
	chmod "u=rwx,go=rx" /mnt/nsg/etc
    else
	eerr "nsg_flashroot(): Mounting tmpfs on '/mnt/nsg/etc' failed."
	l_return_hook 1 0
	return ${EX_UNAVAILABLE}
    fi

# step 2
    findflag "nsgverbose" "true" && \
	einfo "nsg_flashroot(): Extracting config. Please wait..."
    if findflag "nsgdefconfig" "true"; then
	setflag "l_nsgdefconfig" "true"
	findflag "nsgverbose" "true" && \
	    einfo "nsg_flashroot(): Default config is set up manually."
    else
	dd if="$l_nsgconfigdev" bs=64k count=4 | \
	    tar --no-same-owner -xz -f - -C /mnt/nsg/etc
	if [ "$?" -ne "0" ]; then
	    setflag "l_nsgdefconfig" "true"
	    ewarn "nsg_flashroot(): NSG config is corrupt, trying default config."
	fi
    fi
    
    if findflag "l_nsgdefconfig" "true"; then
	ewarn "nsg_flashroot(): Extracting default config. Please wait..."
	tar --no-same-owner -xz -f /mnt/nsg/mnt/defetc.tar.gz -C /mnt/nsg/etc
	if [ "$?" -ne "0" ]; then
	    eerr "nsg_flashroot(): NSG default config is corrupt or not found."
	    l_return_hook 2 1 0
	    return ${EX_UNAVAILABLE}
	fi
    fi
    
    [ "${nsgverbose}" = "true" ] && 
    einfo "nsg_flashroot(): Copying common files. Please wait..."
    tar -c -f - -C /lib . | tar --no-same-owner -x -f - -C /mnt/nsg/lib
    if [ "$?" -ne "0" ]; then
	eerr "nsg_flashroot(): Copying libraries to '/mnt/nsg/lib' failed."
	l_return_hook 2 1 0
	return ${EX_UNAVAILABLE}
    fi
    tar -c -f - -C /bin busybox | tar --no-same-owner -x -f - -C /mnt/nsg/bin
    if [ "$?" -ne "0" ]; then
	eerr "nsg_flashroot(): Copying 'busybox' to '/mnt/nsg/bin' failed."
	l_return_hook 2 1 0
	return ${EX_UNAVAILABLE}
    fi

    if mount -t tmpfs none /mnt/nsg/var; then
	chmod "ugo=rx" /mnt/nsg/proc
	chmod "u=rwx,go=rx" /mnt/nsg/var
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/home
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/home/user
	chown 1000:1000 /mnt/nsg/var/home/user
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/empty
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/lock
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/log
	mkdir -m "u=rwx,go=rx" /mnt/nsg/var/run
	mkdir -m "a=rwx" /mnt/nsg/var/tmp
    else
	eerr "nsg_flashroot(): Mounting tmpfs on '/mnt/nsg/var' failed."
	l_return_hook 2 1 0
	return ${EX_UNAVAILABLE}
    fi

# step 3
    if /sbin/depmod -a -b /mnt/nsg; then
	:
    else
	ewarn "nsg_flashroot(): Depmod can't create dependency file."
    fi

    if mount -o remount,ro none /mnt/nsg; then
	:
    else
	eerr "nsg_flashroot(): Remounting tmpfs read-only on '/mnt/nsg' failed."
	l_return_hook 3 2 1 0
	return ${EX_UNAVAILABLE}
    fi

    if [ -e /proc/sys/nsg/.sys/configdev ]; then
	echo -E "${l_nsgconfigdev}" > /proc/sys/nsg/.sys/configdev
    else
	eerr "nsg_flashroot(): Writing to file '/proc/sys/nsg/.sys/configdev' failed."
	l_return_hook 3 2 1 0
	return ${EX_UNAVAILABLE}
    fi

    if [ -e /proc/sys/nsg/.sys/rootdev ]; then
	echo -E "${l_nsgrootdev}" > /proc/sys/nsg/.sys/rootdev
    else
	eerr "nsg_flashroot(): Writing to file '/proc/sys/nsg/.sys/rootdev' failed."
	l_return_hook 3 2 1 0
	return ${EX_UNAVAILABLE}
    fi
    
    return ${EX_OK}
}

nsg_nfsroot() {
    local l_nsgrootdev l_nfs_root_source

    if l_nsgrootdev="$(nsg_getrootdev "nfs")"; then
	:
    else
	eerr "nsg_nfsroot(): Can't get rootfs device."
	return ${EX_UNAVAILABLE}
    fi

    l_nfs_root_source=`udhcpc -q | \
	tee | \
	grep -e '^nfs_root_source=' | \
	sed -n -e 's/^nfs_root_source=//p`

    if [ -z "${l_nfs_root_source}" ]; then
	eerr "nsg_nfsroot(): Can't get nfs serverid or rootpath."
	return ${EX_UNAVAILABLE}
    fi

    if mount -o nolock,ro ${l_nfs_root_source} /mnt/nsg; then
	mount -o remount,rw ${l_nfs_root_source} /mnt/nsg
    else
	eerr "nsg_nfsroot(): Mounting rootfs on '/mnt/nsg' failed."
	return ${EX_UNAVAILABLE}
    fi
    
    if /sbin/depmod -a -r -b /mnt/nsg; then
	:
    else
	ewarn "nsg_nftlroot(): Depmod can't create dependency file."
    fi

    if [ -e /proc/sys/nsg/.sys/rootdev ]; then
	echo -E "${l_nsgrootdev}" > /proc/sys/nsg/.sys/rootdev
    else
	ewarn "nsg_nfsroot(): File '/proc/sys/nsg/.sys/rootdev' not found."
    fi

    return ${EX_OK}
}
