#!/bin/echo It can be only inclusion. Incorrect start

# Use this for include.
# [ -z "${_NSG_REVISION_UPDATE}" ] && { . /usr/local/share/nsg/revision/revision_dialog.sh || do_it_if_error; }

_NSG_REVISION_UPDATE=1

[ -z "${_NSG_FUNCTIONS}" ] && { . /usr/local/share/nsg/lib/functions.sh || return 69; }
[ -z "${_NSG_DIALOG}" ] && { . /usr/local/share/nsg/lib/dialog.sh || return 69; }

nsg_revision_update_welcome() {
    l_usage() {
	einfo "nsg_revision_update_welcome(): function usage: nsg_revision_dialog_welcome [-t timeout] [-d default_choice] var"
    }

    l_restore_tty() {
	[ -n "${l_old_tty_settings}" ] && stty "${l_old_tty_settings}"
    }

    l_return_hook() {
	l_restore_tty
    }

    local l_i l_name l_old_tty_settings
    local l_defchoice l_timeout

    l_timeout="5"
    l_defchoice="1"

    # save tty
    l_old_tty_settings="$(stty -g)"
    stty -echo
    
    while getopts t:d: l_name; do
	case $l_name in
	    (t)
	    l_timeout="$OPTARG"
	    ;;
	    (d)
	    l_defchoice="$OPTARG"
	    if echo "${l_defchoice}" | grep -qv -e '^[01234]$'; then
		eerr "nsg_revision_update_welcome(): '${l_defchoice}': default_choice argument must one of '0 1 2 3 4'"
		l_usage
		l_return_hook
		return ${EX_USAGE}
	    fi
	    ;;
	    (?)
	    l_usage
	    l_return_hook
	    return ${EX_USAGE}
	    ;;
	esac
    done
    shift $(($OPTIND - 1)); OPTIND="1"

    if [ "$#" -ne "1" ]; then
	eerr "nsg_revision_update_welcome(): '$@': incorrect quantity of operands."
	l_usage
	l_return_hook
	return ${EX_USAGE}
    fi

    echo ""
    echo "Update menu."
    echo ""
    echo "1) NFTL (DoC)"
    echo "2) sumo (onboard)"
    echo "3) FLEX"
    echo "4) USB flash"
    echo "0) exit"
    echo ""
    echo "timeout='${l_timeout}', default_choice='${l_defchoice}'"
    echo -n "Take your choice: "

    while nsg_wait_getkey -t "${l_timeout}" -d "${l_defchoice}" -- "$@"; do
	echo ""
	if eval echo \"\$$@\" | grep -qv -e '^[01234]$' -e '^\\n$'; then
	    echo -n "Try again: "
	    continue
	else
	    if [ "$(eval echo \"\$$@\")" == "\n" ]; then
		eval "$@"='${l_defchoice}'
	    fi
	    eval echo "\$$@"
	    l_return_hook
	    return ${EX_OK}
	fi
    done
    
    l_return_hook
    return ${EX_UNAVAILABLE}
}

nsg_revision_update_sure() {
    l_usage() {
	einfo "nsg_revision_update_sure(): function usage: nsg_revision_update_sure [-t timeout] [-d default_choice] var"
    }
    
    nsg_dialog_sure $@
}

nsg_revision_update_dialog() {
    local l_defvalue l_value l_surevalue l_defsurevalue
    local l_ic

    l_ic="$1"
    l_defvalue="0"
    while true; do
	nsg_revision_update_welcome -t "25" -d "$l_defvalue" -- "l_value"
	if [ "$l_value" != "$l_defvalue" ]; then
	    l_surevalue=""
	    l_defsurevalue="no"
	    nsg_revision_update_sure -t "25" -d "$l_defsurevalue" -- "l_surevalue" || \
		eerr "nsg_revision_update_dialog(): Internal error."
	    if [ "$l_surevalue" = "no" ]; then
		l_value="$l_defvalue"
		continue
	    fi
	fi
	break
    done
    
    case "$l_value" in
	(0)
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_dialog(): --> exit"
	pushflag "nsgic" "exit"
	;;
	(1)
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_dialog(): --> Update NFTL"
	pushflag "nsgic" "nftl"
	;;
	(2)
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_dialog(): --> Update flash"
	pushflag "nsgic" "flash"
	;;
	(3)
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_dialog(): --> Update FLEX"
	pushflag "nsgic" "flex"
	;;
	(4)
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_dialog(): --> Update USB flash"
	pushflag "nsgic" "usbflash"
	;;
	(*)
	eerr "nsg_revision_update_dialog(): Unknown update type."
	pushflag "nsgic" "unknown"
	;;
    esac
}

[ -z "${_NSG_ROOTFS}" ] && { . /usr/local/share/nsg/lib/rootfs.sh || return 69; }

nsg_revision_update_format() {
    local l_name

    case "$1" in
	(nftl)
#xxxxxx	
	edebug "nsg_revision_update_format(): nftl."
	return 1
	;;
	(*)
	eerr "nsg_revision_update_check(): Unknown target '$1'."
	return ${EX_SOFTWARE}
	;;
    esac
}

nsg_revision_update_check() {
    local l_name

    case "$2" in
	(nftl)
	if l_nsgrootdev="$(nsg_getrootdev "nftl")"; then
	    l_name="$(sed -n -e 's/^[[:space:]]*93[[:space:]]*\(.*\)[[:space:]]*$/\1/p;T;q' /proc/devices)"
	    if [ "$l_name" = "nftl" ]; then
		findflag "nsgverbose" "true" && \
		    einfo "nsg_revision_update_check(): Nftl device is available."
		l_name="$(sed -n -e 's/^[[:space:]]*93[[:space:]]*0[[:space:]]*[[:digit:]]*[[:space:]]*\(.*\)[[:space:]]*$/\/dev\/\1/p;T;q' /proc/partitions)"
		if [ -z "${l_name}" ]; then
		    ewarn "nsg_revision_update_check(): Nftl patrtition needs format."
#xxx
		    edebug "nsg_revision_update_check(): This service is not available."
		    return ${EX_UNAVAILABLE}
#zzz
		    if nsg_revision_update_format "nftl_partition"; then
			ewarn "Update not complete!"
			ewarn "Try update again."
			return ${EX_SOFTWARE}
		    else
			eerr "nsg_revision_update_check(): Nftl partition format failed."
			eerr "May be device crash."
			return ${EX_UNAVAILABLE}
		    fi
		fi
		if [ -b "${l_name}" ]; then
		    pushflag "$1" "${l_name}"
		    return ${EX_OK}
		else
		    eerr "nsg_revision_update_check(): File '${l_name}' not found or is not block special."
		    return ${EX_UNAVAILABLE}
		fi
	    else
		eerr "nsg_revision_update_check(): Block device '93 nftl' not available."
		ewarn "nsg_revision_update_check(): Mtd device needs format."
#xxx
		    edebug "nsg_revision_update_check(): This service is not available."
		    return ${EX_UNAVAILABLE}
#zzz
		if nsg_revision_update_format "nftl"; then
		    ewarn "Update not complete!"
		    ewarn "After restart try update again."
		    ewarn "Rebooting in 5 seconds..."
		    /sbin/reboot -d5
		    return ${EX_SOFTWARE}
		else
		    eerr "nsg_revision_update_check(): Nftl format failed."
		    eerr "May be device crash."
		    return ${EX_UNAVAILABLE}
		fi
		
		
		
		return ${EX_UNAVAILABLE}
	    fi
	else
	    eerr "nsg_revision_update_check(): Can't found DoC device."
	    return ${EX_UNAVAILABLE}
	fi
	;;
	(flash)
	ewarn "nsg_revision_update_check(): This service is not available."
	return ${EX_UNAVAILABLE}
	;;
	(flex)
	ewarn "nsg_revision_update_check(): This service is not available."
	return ${EX_UNAVAILABLE}
	;;
	(usbflash)
	return ${EX_UNAVAILABLE}
	ewarn "nsg_revision_update_check(): This service is not available."
	;;
	(*)
	eerr "nsg_revision_update_check(): Unknown target '$2'."
	return ${EX_SOFTWARE}
	;;
    esac
    
    eerr "nsg_revision_update_check(): Internal error."
    return ${EX_SOFTWARE}
}

nsg_revision_update_nftl() {
    local l_name l_value l_defvalue l_err l_old_tty_settings l_surevalue l_defsurevalue

    if nsg_revision_update_check "nsgic" "nftl"; then
	l_name="$(popflag "nsgic")"
    else
	eerr "nsg_revision_update_nftl(): NFTL not found or contains errors."
	return ${EX_UNAVAILABLE}
    fi

    if mkdir -p /tmp/nsgupdate; then
	:
    else
	eerr "nsg_revision_update_nftl(): Can't create directory '/tmp/nsgupdate'"
	return ${EX_UNAVAILABLE}
    fi
    
    if mount -t tmpfs none /tmp/nsgupdate; then
	:
    else
	eerr "nsg_revision_update_nftl(): Mounting tmpfs on '/tmp/nsgupdate' failed."
	return ${EX_UNAVAILABLE}
    fi
    
    if [ ! -d /mnt/nsg ]; then
	ewarn "nsg_revision_update_nftl(): Mountpoint '/mnt/nsg' not found."
	if mkdir -p /mnt/nsg; then
	    :
	else
	    eerr "nsg_revision_update_nftl(): Can't create directory '/mnt/nsg'."
	    umount -r /tmp/nsgupdate || \
		eerr "nsg_revision_update_nftl(): Unmounting filesystem from '/tmp/nsgupdate' failed."
	    return ${EX_UNAVAILABLE}
	fi
    fi
    
    if mount -t minix -o rw "$l_name" /mnt/nsg; then
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_nftl(): Mounting '$l_name' on '/mnt/nsg' successfully."
    else
	eerr "nsg_revision_update_nftl(): Mounting '$l_name' on '/mnt/nsg' failed."
	echo "Do You want create filesystem on NFTL ('$l_name')?"
	l_value=""
	l_defvalue="no"
	nsg_dialog_yesno -t "25" -d "$l_defvalue" -- "l_value" || \
	    eerr "nsg_revision_update_nftl(): Internal error."
	if [ "$l_value" != "yes" ]; then
	    l_value="$l_defvalue"
	    umount -r /tmp/nsgupdate || \
		eerr "nsg_revision_update_nftl(): Unmounting filesystem from '/tmp/nsgupdate' failed."
	    return ${EX_UNAVAILABLE}
	fi
	if mkfs.minix -v "$l_name"; then
	    einfo "nsg_revision_update_nftl(): Making filesystem on '$l_name' successfully."
	    fsck.minix -a "$l_name"
	    if mount -t minix -o rw "$l_name" /mnt/nsg; then
		findflag "nsgverbose" "true" && \
		    einfo "nsg_revision_update_nftl(): Mounting '$l_name' on '/mnt/nsg' successfully."
	    else
		eerr "nsg_revision_update_nftl(): Mounting '$l_name' on '/mnt/nsg' failed."
		umount -r /tmp/nsgupdate || \
		    eerr "nsg_revision_update_nftl(): Unmounting filesystem from '/tmp/nsgupdate' failed."
		return ${EX_UNAVAILABLE}
	    fi
	else
	    eerr "nsg_revision_update_nftl(): Making filesystem on '$l_name' failed."
	    umount -r /tmp/nsgupdate || \
		eerr "nsg_revision_update_nftl(): Unmounting filesystem from '/tmp/nsgupdate' failed."
	    return ${EX_UNAVAILABLE}
	fi
    fi
    
    echo "Update parameters format:"
    echo -e "\t<client_ip> <netmask> <server_ip> <nftl.tar.gz_file>"
    echo "For instance:"
    echo -e "\t192.168.0.1 255.255.255.0 192.168.0.250 nsglinux-ppc-nftl.tar.gz"

    while true; do
	echo ""
#    echo "timeout='25'"
#    echo -n "Enter parameters: "
#    nsg_wait_readkeys -t "25" -- "l_value"
#    l_value=$(echo "$l_value" | sed -n -e 's/^\(.*\)\\n$/\1/p')
	l_defvalue="192.168.0.1 255.255.255.0 192.168.0.250 nsglinux-ppc-nftl.tar.gz"
	echo "timeout='600', default_choice='${l_defvalue}'"
	l_old_tty_settings="$(stty -g)"
	stty erase ^H
	read -t "600" -p "Enter parameters: " "l_value"
	stty "${l_old_tty_settings}"
	if [ -z "$l_value" ]; then
	    l_value="${l_defvalue}"
	fi
 	echo ""
	set -- $l_value
	echo "client: $1/$2"
	echo "server: $3"
	echo "filename: $4"
	l_surevalue=""
	l_defsurevalue="no"
	nsg_revision_update_sure -t "25" -d "$l_defsurevalue" -- "l_surevalue" || \
	    eerr "nsg_revision_update_dialog(): Internal error."
	if [ "$l_surevalue" = "yes" ]; then
	    break
	fi
	continue
    done

    set -- $l_value
    findflag "nsgverbose" "true" && \
	einfo "nsg_revision_update_nftl(): Running 'ifconfig eth0 $1 netmask $2'..."
    if ifconfig eth0 $1 netmask $2; then
	:
    else
	eerr "nsg_revision_update_nftl(): Running 'ifconfig eth0 $1 netmask $2' failed."
	umount -r /mnt/nsg || \
	    eerr "nsg_revision_update_nftl(): Unmounting file system from '/mnt/nsg' failed."
	umount -r /tmp/nsgupdate || \
	    eerr "nsg_revision_update_nftl(): Unmounting file system from '/tmp/nsgupdate' failed."
	return ${EX_UNAVAILABLE}
    fi
    
    findflag "nsgverbose" "true" && \
	einfo "nsg_revision_update_nftl(): Running 'tftp -g -r $4 -l /tmp/nsgupdate/nftlroot.tar.gz $3'..."
    if tftp -g -r $4 -l /tmp/nsgupdate/nftlroot.tar.gz $3; then
	:
    else
	eerr "nsg_revision_update_nftl(): Running 'tftp -g -r $4 -l /tmp/nsgupdate/nftlroot.tar.gz $3' failed."
	umount -r /mnt/nsg || \
	    eerr "nsg_revision_update_nftl(): Unmounting file system from '/mnt/nsg' failed."
	umount -r /tmp/nsgupdate || \
	    eerr "nsg_revision_update_nftl(): Unmounting file system from '/tmp/nsgupdate' failed."
	return ${EX_UNAVAILABLE}
    fi
    
    l_err="0"
    test -d /mnt/nsg/etc || l_err="2"
    if [ -d /mnt/nsg/mnt ]; then
	mkdir -p /mnt/nsg/mnt || l_err="1"
    fi
    if [ "$l_err" -eq "0" ]; then
	tar -c -f /mnt/nsg/mnt/baketc.tar -C /mnt/nsg/etc . || l_err="1"
    fi
    if [ "$l_err" -eq "0" ]; then
	if [ -f /mnt/nsg/mnt/baketc.tar.gz ]; then
	    if mv /mnt/nsg/mnt/baketc.tar.gz /mnt/nsg/mnt/savetc.tar.gz; then
		gzip -9 /mnt/nsg/mnt/baketc.tar || l_err="1"
		if [ "$l_err" -eq "0" ]; then
		    rm /mnt/nsg/mnt/savetc.tar.gz
		else
		   ewarn "nsg_revision_update_nftl(): Compressing user config failed."
		   mv /mnt/nsg/mnt/savetc.tar.gz /mnt/nsg/mnt/baketc.tar.gz
		fi
	    else
		ewarn "nsg_revision_update_nftl(): Can't save previous stored user config."
		l_err="1"
	    fi
	else
	    gzip -9 /mnt/nsg/mnt/baketc.tar || l_err="1"
	fi
    fi
    if [ "$l_err" -eq "0" ]; then
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_nftl(): Saving user config to 'baketc.tar.gz' successfully."
    elif [ "$l_err" -eq "2" ]; then
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_nftl(): Saving user config not required."
    else
	ewarn "nsg_revision_update_nftl(): Saving user config failed."
    fi

    findflag "nsgverbose" "true" && \
	einfo "nsg_revision_update_nftl(): Extracting rootfs. Please wait..."
    tar --no-same-owner -xz -f /tmp/nsgupdate/nftlroot.tar.gz -C /mnt/nsg
    if [ "$?" -eq "0" ]; then
	if [ "$l_err" -eq "2" ]; then
	    findflag "nsgverbose" "true" && \
		einfo "nsg_revision_update_nftl(): Extracting default config. Please wait..."
	    tar --no-same-owner -xz -f /mnt/nsg/mnt/defetc.tar.gz -C /mnt/nsg/etc
	    if [ "$?" -ne "0" ]; then
		eerr "nsg_revision_update_nftl(): Extracting default config failed."
		umount -r /mnt/nsg || \
		    eerr "nsg_revision_update_nftl(): Unmounting file system from '/mnt/nsg' failed."
		umount -r /tmp/nsgupdate || \
		    eerr "nsg_revision_update_nftl(): Unmounting file system from '/tmp/nsgupdate' failed."
		return ${EX_UNAVAILABLE}
	    fi
	fi
    else
	eerr "nsg_revision_update_nftl(): Extracting rootfs failed."
	umount -r /mnt/nsg || \
	    eerr "nsg_revision_update_nftl(): Unmounting file system from '/mnt/nsg' failed."
	umount -r /tmp/nsgupdate || \
	    eerr "nsg_revision_update_nftl(): Unmounting file system from '/tmp/nsgupdate' failed."
	return ${EX_UNAVAILABLE}
    fi

    findflag "nsgverbose" "true" && \
	einfo "nsg_revision_update_nftl(): Updating successfully."

    umount -r /mnt/nsg || \
	ewarn "nsg_revision_update_nftl(): Unmounting file system from '/mnt/nsg' failed."
    umount -r /tmp/nsgupdate || \
	ewarn "nsg_revision_update_nftl(): Unmounting file system from '/tmp/nsgupdate' failed."
    
    return ${EX_OK}
}

nsg_revision_update_flash() {
    ewarn "nsg_revision_update_flash(): This service is not available."
    return ${EX_UNAVAILABLE}
}

nsg_revision_update_flex() {
    ewarn "nsg_revision_update_flex(): This service is not available."
    return ${EX_UNAVAILABLE}
}

nsg_revision_update_usbflash() {
    ewarn "nsg_revision_update_usbflash(): This service is not available."
    return ${EX_UNAVAILABLE}
}

nsg_revision_update_main() {
    local l_type l_name

    nsg_revision_update_dialog "nsgic"
    l_type="$(popflag "nsgic")"
    
    case "$l_type" in
	(nftl)
	nsg_revision_update_nftl
	return $?
	;;
	(flash)
	nsg_revision_update_flash
	return $?
	;;
	(flex)
	nsg_revision_update_flex
	return $?
	;;
	(usbflash)
	nsg_revision_update_usbflash
	return $?
	;;
	(exit)
	findflag "nsgverbose" "true" && \
	    einfo "nsg_revision_update_main(): Exit from update menu."
	return 1
	(*)
	eerr "nsg_revision_update_main(): Unknown update type '$l_type'."
	return ${EX_SOFTWARE}
	;;
    esac

    eerr "nsg_revision_update_main(): Internal error."
    return ${EX_SOFTWARE}
}
